use actix_web::{get, http::header::ContentType, post, web, HttpResponse, Responder};
use gamenight_database::{game::{insert_game, load_game, rename_game}, DbPool, GetConnection};
use uuid::Uuid;

use crate::{models::{add_game_request_body::AddGameRequestBody, game::Game, game_id::GameId, rename_game_request_body::RenameGameRequestBody}, request::{authorization::AuthUser, error::ApiError}};

#[get("/games")]
pub async fn get_games(pool: web::Data<DbPool>, _user: AuthUser) -> Result<impl Responder, ApiError> {
    let mut conn = pool.get_conn();
    let games: Vec<gamenight_database::game::Game> = gamenight_database::games(&mut conn)?;
    let model: Vec<Game> = games.iter().map(|x| {
        Game {
            id: x.id.to_string(),
            name: x.name.clone(),
        }}
    ).collect();

    Ok(HttpResponse::Ok()
        .content_type(ContentType::json())
        .body(serde_json::to_string(&model)?)
    )
}

impl From<AddGameRequestBody> for gamenight_database::game::Game {
    fn from(value: AddGameRequestBody) -> Self {
        Self {
            id: Uuid::new_v4(),
            name: value.name
        }
    }
}

#[get("/game")]
pub async fn get_game(pool: web::Data<DbPool>, _user: AuthUser, game_id: web::Json<GameId>) -> Result<impl Responder, ApiError> {
    let mut conn = pool.get_conn();
    let db_game = load_game(&mut conn, Uuid::parse_str(&game_id.0.game_id)?)?;

    let model = Game { 
        id: db_game.id.to_string(),
        name: db_game.name
    };

    Ok(HttpResponse::Ok()
        .content_type(ContentType::json())
        .body(serde_json::to_string(&model)?)
    )
}

#[post("/game")]
pub async fn post_game(pool: web::Data<DbPool>, _user: AuthUser, game_data: web::Json<AddGameRequestBody>) -> Result<impl Responder, ApiError> {
    let mut conn = pool.get_conn();
    insert_game(&mut conn, game_data.0.into())?;

    Ok(HttpResponse::Ok())
}

#[post("/rename_game")]
pub async fn post_rename_game(pool: web::Data<DbPool>, _user: AuthUser, game_data: web::Json<RenameGameRequestBody>) -> Result <impl Responder, ApiError> {
    let mut conn = pool.get_conn();
    rename_game(&mut conn, Uuid::parse_str(&game_data.0.id)?, game_data.0.name)?;

    Ok(HttpResponse::Ok())
}