
use gamenight_api_client_rs::{apis::default_api::game_get, models::GameId};
use inquire::Select;

use crate::{domain::game::Game, flows::{exit::Exit, rename_game::RenameGame}};

use super::*;

#[derive(Clone)]
pub struct ViewGame {
    game: Game
}

impl ViewGame {
    pub fn new(game: Game) -> Self {
        Self {
            game
        }
    }
}

#[async_trait]
impl<'a> Flow<'a> for ViewGame {
    async fn run(&self, state: &'a mut GamenightState) -> FlowResult<'a> {
        let game_id = GameId{ game_id: self.game.id.to_string() };
        let game: Game = game_get(&state.api_configuration, Some(game_id)).await?.into();

        println!("{}", game);

        let options: Vec<Box<dyn Flow<'a> + Send>> = vec![
            Box::new(RenameGame::new(game.clone())),
            Box::new(Exit::new())
        ];
        let choice = Select::new("What do you want to do:", options)
            .prompt_skippable()?;

        self.continue_choice(state, &choice).await
    }
}

impl Display for ViewGame {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", &self.game.name)
    }
}

